function [] = runme()
  %RUNME Entry point for assignment 4
  %   [] = RUNME() executes the various steps needed to classify images. 
  %
  
  % Directories for train/test images. The directory contains a subdirectory for each image class.
  % The directory name is the same as the class name. Within each subdirectories there are images
  % belonging to that specific class.
  BASEPATH_TRAIN = '../train_input/';
  BASEPATH_TEST = '../test_input/';

  % Get the translation table from integers to class names. We assume BASEPATH_TRAIN and 
  % BASEPATH_TEST contain the same classes. The name of class ii is class_names{ii}.
  class_names = getclassesanddirs(BASEPATH_TRAIN);
  
  % Set specific options used by the different algorithms.
  % For example:
  options.num_patches = 100;
  options.num_clusters = 100;
  
  % Training phase.
  train_results = train(BASEPATH_TRAIN, class_names, options);               

  % Classification phase
  confusion_mat = classify(BASEPATH_TEST, class_names, train_results, options);
  
  % Output phase
  generateoutput(class_names, confusion_mat);
end


function [class_names, class_dirs] = getclassesanddirs(basepath)
  class_names = cell(0, 1);
  class_dirs = cell(0, 1);
  
  % Each sub directory of basepath defines a class. We extract all subdirectory names into 
  % tmp_names.
  tmp_names = dir(basepath);
  tmp_names = tmp_names([tmp_names.isdir]);
  tmp_names = {tmp_names.name}';
  
  % Retrieve class names and paths
  for ii=1:numel(tmp_names)
    % do not copy '.' and '..' and any folder that starts with '.'
    dot_locations = strfind(tmp_names{ii}, '.');
    if (isempty(dot_locations) || (dot_locations(1) ~= 1))
      class_names(end + 1, 1) = tmp_names(ii);
      class_dirs{end + 1, 1} = [basepath tmp_names{ii} '/'];
    end
  end
end



function [] = generateoutput(class_names, confusion_mat)

  output_filename_prefix = ['../output/result'];
  
  % Create output file with success rates
  output_filename_txt = [output_filename_prefix '.txt'];
  fid = fopen(output_filename_txt, 'w');
  overall_success_rate = trace(confusion_mat) ./ sum(confusion_mat(:));
  fprintf(fid, 'Overall success rate: %1.2f\n\n', overall_success_rate);
  class_success_rates = diag(confusion_mat) ./ sum(confusion_mat, 1)';
  fprintf(fid, 'Individual success rates:\n');
  for ii = 1:numel(class_names)
    fprintf(fid, '%-13s %1.2f\n', [class_names{ii} ':'], class_success_rates(ii));
  end
  fclose(fid);

  % Create output image. Good results should be brighter on the diagonal.
  output_filename_img = [output_filename_prefix '.jpg'];
  h = figure('Visible', 'Off');
  imagesc(confusion_mat);
  colorbar;
  print(h, '-djpeg', '-r300', output_filename_img);
  close(h);
end