function [features] = sift_detectfeatures(input_image, max_features)
%SIFT_DETECTFEATURES Generate sift features corresponding to salient points of an image.
%   [FEATURES] = SIFT_DETECTFEATURES(INPUT_IMAGE, MAX_FEATURES) takes as input a grayscale image,
%   detects salient points in the image and generates features to represent those points, using the
%   sift method.
%
%   input parameters:
%       INPUT_IMAGE  - PxQ matrix representing an image.
%       MAX_FEATURES - Number of features to return. 'inf' will return all detected features.
%
%   output parameters:
%       FEATURES - 4xF matrix. Each column contains x-location, y-location, scale and orientation.
%                  For example, locations(2, 5) is the y coordinate of the 5th salient point.
%                  location(3, 10) is the scale of the 10th point.
%                  F equals the number of detected features or max_features, the smaller of the two.
%

if ~exist('sift.m', 'file')
    addpath('sift');
end

[features, ~] = sift(input_image);
if (~isinf(max_features))
  features(:, (max_features + 1):end) = [];
end

end

